/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

const DB_CONFIG_URL = "chrome://thumbsdown/content/sql/sqlConfig.js";


var Application = Cc["@mozilla.org/fuel/application;1"].getService(Ci.fuelIApplication);
var storageSvc = Cc["@mozilla.org/storage/service;1"].getService(Ci.mozIStorageService);

Cu.import("resource://gre/modules/XPCOMUtils.jsm");


function Database() { /* empty */ }

Database.prototype =
{
    classID: Components.ID("{698b2b8c-2bb4-482a-85db-473452d75551}"),
    
    QueryInterface: XPCOMUtils.generateQI(
        [Ci.nsIObserver,
         Ci.nsISupports,
         Ci.nsISupportsWeakReference]),
    
    initialize: function()
    {
        var loader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
        var sqlConfig = {}
        loader.loadSubScript(DB_CONFIG_URL, sqlConfig);
        sqlConfig = sqlConfig.sqlConfig;
        
        var dbFile = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
        dbFile.append(sqlConfig.databaseFilename);
        var exists = dbFile.exists();
        var db = storageSvc.openDatabase(dbFile);
        
        db.beginTransaction();
        try
        {
            if (! exists)
            {
                var scriptName = sqlConfig.scriptBaseUrl + "schema.sql";
                this._executeScript(db, scriptName);
            }
            else
            {
                if (db.schemaVersion > sqlConfig.schemaVersion)
                {
                    throw new Components.Exception("database " + sqlConfig.databaseFilename + " schema version (" + db.schemaVersion + ") is unknown");
                }
                else if (db.schemaVersion < sqlConfig.schemaVersion)
                {
                    if (sqlConfig.backupDatabase)
                        this._backupDatabase(db);
                    
                    while (db.schemaVersion != sqlConfig.schemaVersion)
                    {
                        var scriptName = sqlConfig.scriptBaseUrl + "migrate-" + db.schemaVersion + ".sql";
                        this._executeScript(db, scriptName);
                    }
                }
            }
            
            db.commitTransaction();
            db.close();
        }
        catch (e)
        {
            db.rollbackTransaction();
            db.close();
            
            if (! exists)
                dbFile.remove();
            
            throw e;
        }
    },
    
    _executeScript: function(db, url)
    {
        Application.console.log("running sql script: " + url);
        
        for each (var stmt in this._loadStatements(url))
            db.executeSimpleSQL(stmt);
    },
    
    _loadStatements: function(url)
    {
        var script = this._loadScript(url);
        script = script.replace(/--.*/g, "");
        script = script.replace(/[\n\r]/g, " ");
        
        var statements = script.split(";");
        statements = [s.replace(/^\s+|\s+$/g, "").replace(/\s\s+/g, " ") for each (s in statements)];
        statements = statements.filter(function(s) { return (s); });
        
        return statements;
    },
    
    _loadScript: function(url)
    {
        var ioSvc = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
        var stream = Cc["@mozilla.org/scriptableinputstream;1"].getService(Ci.nsIScriptableInputStream);
        var channel = ioSvc.newChannel(url, null, null);
        var input = channel.open();
        stream.init(input);
        var content = stream.read(input.available());
        stream.close();
        input.close();
        return content;
    },
    
    _backupDatabase: function(db)
    {
        var dbFile = db.databaseFile;
        var dbFilename = dbFile.leafName;
        var backupFilename = dbFilename + "." + db.schemaVersion + ".bak";
        var backupFile = storageSvc.backupDatabaseFile(dbFile, backupFilename);
        Application.console.log("making backup before database migration: " + dbFilename + " -> " + backupFile.leafName);
        return backupFile;
    },
    
    // nsIObserver
    observe: function(subject, topic, data)
    {
        switch (topic)
        {
            case "profile-after-change":
                this.initialize();
                break;
        }
    }
}


const NSGetFactory = XPCOMUtils.generateNSGetFactory([Database]);
