/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


const Cc = Components.classes;
const Ci = Components.interfaces;

const TOPIC_NEW     = "thumbsdown-script-new";
const TOPIC_DESTROY = "thumbsdown-script-destroy";
const TOPIC_CHANGE  = "thumbsdown-script-change";

const EVENT_HANDLERS = {}
EVENT_HANDLERS[TOPIC_NEW]     = onScriptNew;
EVENT_HANDLERS[TOPIC_DESTROY] = onScriptDestroy;
EVENT_HANDLERS[TOPIC_CHANGE]  = onScriptChange;

var scriptRunner;
var tree;
var observer;
var isContextMenuOpen = false;
var updateTimer;


function onLoad()
{
    stringBundle = $("stringbundle");
    scriptRunner = Cc['@thumbsdown.mozdev.org/script-runner;1'].getService(Ci.nsISupports).wrappedJSObject;
    tree = $("tree_scripts");
    tree.view = new ScriptRunnerTreeView(scriptRunner);
    observer = new ScriptRunnerObserver();
    
    // update timer
    var updateDelay = nsPreferences.getIntPref("extensions.thumbsdown.scriptrunner.updateDelay");
    if (updateDelay > 0)
    {   
        updateTimer = new Timer(false, updateDelay, null, onUpdateTimer);
        updateTimer.start();
    }
    
    // init ui
    _update();
}

function onUnload()
{
    observer.unregister();
    if (updateTimer)
        updateTimer.stop();
}

function onUpdateTimer()
{
    // TODO: invalidate only running scripts
    var col = tree.columns["tcol_elapsed"];
    tree.treeBoxObject.invalidateColumn(col);
}

function onTreeSelect()
{
    _updateToolbar();
}

function onTooltipShowing(e)
{
    if (isContextMenuOpen)
        return false;
    var row = _getTreeRowAt(e.clientX, e.clientY);
    if (row == -1)
        return false;
    var script = scriptRunner.scripts[row];
    $("row_thumbnail").hidden = script.parent;
    $("row_link").hidden = script.parent;
    for each (var name in ["name", "status", "elapsed", "document",  "thumbnail", "link", "image"])
        if (! $("row_" + name).hidden)
            $("descr_" + name).value = _getScriptPropertyAsText(script, name);
    $("thumbnail").src = (script.parent ? script.parent.thumbnailUrl : script.thumbnailUrl);
    return true;
}

function onTooltipHidden()
{
    $("thumbnail").src = "";
}

function onContextMenuShowing(e)
{
    var row = _getTreeRowAt(e.clientX, e.clientY);
    if (row == -1)
        return false;
    tree.view.selection.select(row);
    isContextMenuOpen = true;
    return true;
}

function onContextMenuHidden()
{
    isContextMenuOpen = false;
}

function onScriptNew(scriptRunner, oldLength)
{
    oldLength = Number(oldLength);
    var count = scriptRunner.length - oldLength;
    tree.treeBoxObject.rowCountChanged(oldLength, count);
    updateTimer.trigger();
    schedule(null, _update);
}

function onScriptDestroy(scriptRunner, indexes)
{
    indexes = eval(indexes);
    var selection = tree.view.selection;
    selection.selectEventsSuppressed = true;
    tree.treeBoxObject.beginUpdateBatch();
    for each (var index in indexes)
        tree.treeBoxObject.rowCountChanged(index, -1);
    tree.treeBoxObject.endUpdateBatch();
    selection.selectEventsSuppressed = false;
    updateTimer.trigger();
    schedule(null, _update);
}

function onScriptChange(script)
{
    // TODO: invalidate only changed cells
    tree.treeBoxObject.invalidateRow(script.index);
    if (script.parent && (script.parent.index != undefined))
        tree.treeBoxObject.invalidateRow(script.parent.index);
    
    if (script.isError && nsPreferences.getBoolPref("extensions.thumbsdown.scriptrunner.attention"))
        window.getAttention();
    
    updateTimer.trigger();
    schedule(null, _update);
}

function startScript()
{
    var script = _getSelectedScript();
    if (script)
        script.start();
}

function stopScript()
{
    var scriptIterator = _getSelectedScriptIterator();
    var scripts = [script for each (script in scriptIterator)];
    for each (var script in scripts)
        if (script.isRunning)
            script.cancel();
}

function removeScript()
{
    var scriptIterator = _getSelectedScriptIterator();
    var scripts = [script for each (script in scriptIterator)];
    for each (var script in scripts)
        if (script.isRunning)
            return;
    var indexes = [script.index for each (script in scripts)];
    scriptRunner.removeScripts(indexes);
}

function selectAll()
{
    tree.view.selection.selectAll();
}

function deselectAll()
{
    tree.view.selection.clearSelection();
}

function openDocument()
{
    var script = _getSelectedScript();
    if (! script)
        return;
    var url = script.documentUrl;
    openUrl(url);
}

function openLink()
{
    var script = _getSelectedScript();
    if (! script)
        return;
    var url = script.linkUrl;
    openUrl(url);
}

function _getSelectedScript()
{
    var selection = tree.view.selection;
    if (selection.count != 1)
        return null;
    return scriptRunner.scripts[tree.currentIndex];
}

function _getSelectedScriptIterator()
{
    var selection = tree.view.selection;
    for each (var index in getTreeSelectionIterator(selection))
        yield scriptRunner.scripts[index];
}

function _getTreeRowAt(x, y)
{
    var row = {};
    var col = {};
    var childElement = {};
    tree.treeBoxObject.getCellAt(x, y, row, col, childElement);
    return row.value;
}

function _update()
{
    _updateTitle();
    _updateToolbar();
}

function _updateTitle()
{
    // TODO: cache title and prefix - there's no point in reading them everytime
    var title = _("window.title");
    if (scriptRunner.length > 0)
    {
        var titlePrefix = nsPreferences.copyUnicharPref("extensions.thumbsdown.scriptrunner.titlePrefix");
        var replacements = {
            running: scriptRunner.count.running,
            cancelled: scriptRunner.count.cancelled,
            success: scriptRunner.count.success,
            error: scriptRunner.count.error,
            fail: scriptRunner.count.cancelled + scriptRunner.count.error,
            total: scriptRunner.length
        };
        title = replace(titlePrefix, replacements) + title;
    }
    document.title = title;
}

function _updateToolbar()
{
    var selection = tree.view.selection;
    switch (selection.count)
    {
        case 0:
            $("cmd_startScript").setAttribute("disabled", true);
            $("cmd_stopScript").setAttribute("disabled", true);
            $("cmd_removeScript").setAttribute("disabled", true);
            break;
        case 1:
            var script = scriptRunner.scripts[tree.currentIndex];
            $("cmd_startScript").setAttribute("disabled", (! script.isQueued));
            $("cmd_stopScript").setAttribute("disabled", (! script.isRunning));
            $("cmd_removeScript").setAttribute("disabled", script.isRunning);
            break;
        default:
            // is one or more scripts running in the selection
            var isRunning = false;
            var scriptIterator = _getSelectedScriptIterator();
            for each (var script in scriptIterator)
                if (script.isRunning)
                {
                    isRunning = true;
                    break;
                }
            $("cmd_startScript").setAttribute("disabled", true);
            $("cmd_stopScript").setAttribute("disabled", (! isRunning));
            $("cmd_removeScript").setAttribute("disabled", isRunning);
            break
    }
}

function _getScriptPropertyAsText(script, name)
{
    switch (name)
    {
        case "name":      return script.name;
        case "status":    return _getStatus(script);
        case "elapsed":   return _getElapsed(script.startTime, script.endTime);
        case "document":  return script.documentUrl;
        case "thumbnail": return script.thumbnailUrl;
        case "link":      return script.linkUrl;
        case "image":     return (script.child ? script.child.result : script.result);
        default:
            throw new Error("unknown name: " + name);
    }
}

function _getStatus(script)
{
    switch (script.state)
    {
       case 0: return _("status.queued");
       case 1: return _("status.evaluating", script.scriptState);
       case 2: return _("status.loading", script.scriptState);
       case 3: return _("status.callback");
       case 4: return _("status.cancelled");
       case 5: return _("status.success") + (script.child ? " > " + _getStatus(script.child) : "");
       case 6:
           var oldStatus = this._getStatus({state: script.oldState, scriptState: script.scriptState});
           return _("status.error", oldStatus, script.error);
       default:
           throw new Error("invalid state:" + script.state);
    }
}

function _getElapsed(start, end)
{
    if (! start)
        return null;
    start = start.getTime();
    end = (end ? end.getTime() : Date.now());
    var time = "" + Math.round((end - start) / 100);
    if (time.length == 1)
        return "0." + time + "s";
    var i = time.length - 1;
    return time.substr(0, i) + "." + time.charAt(i) + "s";
}


function ScriptRunnerObserver()
{
    this.register();
}

ScriptRunnerObserver.prototype =
{
    observe: function(subject, topic, data)
    {
        if (! (topic in EVENT_HANDLERS))
            throw new Error("unknown topic: " + topic);
        EVENT_HANDLERS[topic](subject.wrappedJSObject, data);
    },
    
    register: function()
    {
        var obsSvc = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
        for (topic in EVENT_HANDLERS)
            obsSvc.addObserver(this, topic, false);
    },
    
    unregister: function()
    {
        var obsSvc = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
        for (topic in EVENT_HANDLERS)
            obsSvc.removeObserver(this, topic);
    }
}


// TODO: cache cell texts? no direct access to underlying model
function ScriptRunnerTreeView(scriptRunner)
{
    this.scripts = scriptRunner.scripts;
}

ScriptRunnerTreeView.prototype =
{
    get rowCount() { return this.scripts.length; },
    
    setTree: function(treeBox)
    {
        this.treeBox = treeBox;
    },
    
    getCellText: function(row, col)
    {
        var index = col.id.lastIndexOf("_");
        var name = col.id.substr(index + 1);
        var script = this.scripts[row];
        return _getScriptPropertyAsText(script, name);
    },
    
    isContainer: function(row) { return false; },
    isSeparator: function(row) { return false; },
    isSorted: function() { return false; },
    getLevel: function(row) { return 0; },
    getImageSrc: function(row, col) { return null; },
    getRowProperties: function(row, props) {},
    getCellProperties: function(row, col, props) {},
    getColumnProperties: function(colid, col, props) {}
}
